--*******************************************************************************************************************************
--
-- Description: This module creates a sample count. 
--					
-- Rev History: 3-31-2014:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library UNISIM; 
use UNISIM.Vcomponents.all; 

use work.model_specific_package.all;
use work.transceiver_package.all;

entity rx_counter is
port (
    
    -- Clock and reset generated by the PLDA EZDMA core.
    clk				: in std_logic;
    rst				: in std_logic;
    
    test_rddata		: out std_logic_vector(AXI_WIDTH-1 downto 0); -- Data bus carrying an 8 samples when AXI_WIDTH is 128 bits or 4 samples when AXI_WIDTH is 64 bits.  
    count_data		: out std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0) -- Data bus carrying a count to the adc interface. 
);
end rx_counter;

architecture structural of rx_counter is
	
	signal rddata_counter	: unsigned(15 downto 0);
	signal data_counter		: unsigned(15 downto 0);
	signal data_counter_R	: unsigned(15 downto 0);

begin
	
	ONE_COUNT_PER_CLK : if RX_ITEM_PER_CLK = 1 generate
	process (clk,rst)
	  begin
	  	if rst = '1' then                      
	  		data_counter <= (others => '0');
		elsif rising_edge (clk) then
			data_counter <= data_counter + x"0001";
	     end if;
	  end process;
	  
	  count_data 	 <= std_logic_vector(data_counter((RX_ITEM_SIZE)-1 downto 0));
	  	
	end generate;
    
	TWO_COUNT_PER_CLK : if RX_ITEM_PER_CLK = 2 generate
	process (clk,rst)
    begin
    	if rst = '1' then
    		data_counter <= (others => '0');
		elsif rising_edge(clk) or falling_edge(clk) then
			data_counter <= data_counter + x"0002";
       end if;
    end process;
    
	data_counter_R <= data_counter + x"0001";
    count_data 	 <= std_logic_vector(data_counter_R(RX_ITEM_SIZE-1 downto 0)) & std_logic_vector(data_counter(RX_ITEM_SIZE-1 downto 0));
	
    end generate;
    
	process (clk,rst)
    begin
    	if rst = '1' then
    		rddata_counter <= (others => '0');
    		test_rddata <= (others => '0');
		elsif rising_edge (clk) then
			rddata_counter <= rddata_counter + x"0008";
			for i in 0 to AXI_WIDTH/16-1 loop
				test_rddata((15+i*16) downto i*16) <= std_logic_vector(rddata_counter+i);
			end loop;
       end if;
    end process;
	    

end structural;
